/*****************************************************************************
*
*       (c) Copyright Kofax Image Products, 1990-1992.  All rights reserved.
*               Unauthorized use, duplication, or distribution is strictly prohibited.
*
******************************************************************************
*
*                File:  kut.h
*
*         Purpose:      Kofax utilities
*
*       $Revision:   1.41  $
*               $Date:   15 Oct 1992 10:35:42  $
*
*   $NoAuthor:$
*
*      $NoLog:$
*
*****************************************************************************/

#if !defined(kut_h)
#define kut_h

#include <string.h>

/*****************************************************************************
*       Constants
*****************************************************************************/

/***** Symbols used to understand return value from KUTGetCpuType() */

#define KUT_CPU_TYPE_MASK       0x0700
#define KUT_CPU_8086            0x0000
#define KUT_CPU_80186           0x0100
#define KUT_CPU_80286           0x0200
#define KUT_CPU_80386           0x0300
#define KUT_CPU_80486           0x0400
#define KUT_CPU_PROTECTED       0x0800

/*****************************************************************************
*       Data types
*****************************************************************************/

/***** Scaling and rotation basics */

typedef struct tagXYSCALE {     
        union { 
                struct {        
#ifdef HIGH_ENDIAN
                        BYTE n;
                        BYTE m;
#else
                        BYTE m;
                        BYTE n;
#endif
                } nm;
                WORD scale;
        } X, Y;
} XYSCALE, *PXYSCALE, NEAR *NPXYSCALE, FAR *LPXYSCALE;


/***** Parameter descriptors */

#define KUT_READ_ONLY_FLAG                      0x0001

typedef enum {
        Word,
        Dword,
        WordPtr,
        DwordPtr,
        StructPtr,
} KUT_PARAM_ENUM;

typedef union {
        WORD            asWord;
        DWORD           asDword;
        LPWORD          asWordPtr;
        LPDWORD         asDwordPtr;
        LPVOID          asStructPtr;
} KUT_PARAM;

typedef struct {
        KUT_PARAM_ENUM  ParamType;
        KUT_PARAM               Param;
        WORD                    wSize;
        WORD                    wFlags;
} KUT_PARAM_DESC, FAR *LP_KUT_PARAM_DESC;


/***** Memory buffer manipulation data structures */

typedef enum {                                          /* Buffer descriptor */
        KUT_LP                                  = 1,    /* Handle is a pointer to buffer */
        KUT_ORCA_HANDLE                 = 2,    /* Handle is to image on KF board */
        KUT_EMS_HANDLE                  = 3,    /* Handle is to image in EMS */
        KUT_DUAL_MODE_HANDLE    = 4             /* Handle is a Windows 3.0 dual handle */
} KUT_HANDLE_ENUM;

typedef union {
        LPVOID          asLP;                                   /* pointer to buffer */
        WORD            asOrcaHandle;                   /* image handle (buf on KF board) */
        WORD            asEmsHandle;                    /* EMS buffer handle */
        struct {
                LPVOID  protectedModePtr;               /* protected mode pointer */
                WORD    wDOSSegmentSelector;    /* DOS real mode segment selector */
        } asDualModeHandle;                                     /* Windows 3.0 dual handle */
} KUT_HANDLE;

typedef struct {
        KUT_HANDLE_ENUM         HandleType;             /* type of buffer handle */
        KUT_HANDLE                      BufHandle;              /* buffer handle */
        DWORD                           dwBytes;                /* number of bytes in the buffer */
} KUT_BUF_DESC, FAR *LP_KUT_BUF_DESC;


/***** Generic lookup table data structures */

typedef struct {                                                /* For use with KUTTableLookup() */
        WORD    wEntry1;
        WORD    wEntry2;
} KUT_WORD_TABLE, FAR *LP_KUT_WORD_TABLE;


/***** Mapping data structures */

typedef struct tag_KutMapping FAR       *LP_KUT_MAPPING;

typedef struct tag_KutMappingParms {
        short int       srcW;                           /* width of original image */
        short int       srcH;                           /* height of original image */
        RECT            rcSrc;                          /* source rectangle */
        XYSCALE         xyScale;                        /* scale factors to be applied */
        short int       theta;                          /* angle of rotation:
                                                                                        one of 0, 90, 180, 270. */
        POINT           ptDst;                          /* destination point of the upper-left hand
                                                                                        corner of the scaled, rotated,
                                                                                        normalized, and shifted image of
                                                                                        the source rectangle. */
        short int       xOffset;                        /* x scrolling offset to be applied to the
                                                                                        scaled, rotated, normalized and
                                                                                        shifted image of a point. */
        short int       yOffset;                        /* y scrolling offset to be applied to the
                                                                                        scaled, rotated, normalized and
                                                                                        shifted image of a point. */
} KUT_MAPPINGPARMS,     FAR *LP_KUT_MAPPINGPARMS;

/***** Operations available with KUTBlt */

typedef enum {
        KUTBLT_COPY =                   0x0008,         /* Straight bit blt */
        KUTBLT_NOTCOPY =                0x0108,         /* Invert bits during blt */
        KUTBLT_REVERSE_LINES =  0x4000,         /* Bit blt, reverse lines */
} KUTBLT_OP;

/***** Data structure used by the KUTAlarm*() functions.  This struct */
/***** should not be accessed directly, only provided to the KUTAlarm*() */
/***** functions for manipulation.  It is included here only so that it */
/***** may be put into contexts and thereby avoid having to dynamically */
/***** allocate the memory at run-time. */

typedef struct {
        DWORD           dwStartTime;
        DWORD           dwTimeout;
        BOOL            fAlarmSet;
        BOOL            fAlarmTicking;
        BOOL            fAlarmRetry;
} KUT_ALARM, FAR *LP_KUT_ALARM;

/***** Basic functions that are not supported by the C5.1 runtime libraries */
/***** because they need to be model-independent.  The C6.0 compiler supports */
/***** these functions with _f*() functions (e.g. _fmemcpy() ), but in order */
/***** to support customers still using C5.1, the C6.0 runtime library */
/***** functions are not used.  */

/* #define              KUTMemCpy               _fmemcpy */
/* #define              KUTMemSet               _fmemset */
/* #define              KUTStrNCpy              _fstrncpy */
/* #define              KUTStrLen               _fstrlen */
/* #define              KUTStrCpy               _fstrcpy */
/* #define              KUTStrCmp               _fstrcmp */

/* #define              KUTStrNCpy              KOSStrNCpy */
/* #define              KUTStrLen               KOSStrLen */

/*****************************************************************************
**                                                                                                                                                      **
**                      Mathematical/Geometric manipulation functions                                   **
**                                                                                                                                                      **
*****************************************************************************/

/*****************************************************************************
* Function:     KUTFindFitToRectScaleFactors
* Purpose:      Determine scale factors so that the scaled image of the source
*                       rectangle will be a "best fit" to the destination rectangle.  The
*                       scaled image of the source rectangle will always lie inside the
*                       destination rectangle.
* Returns:
*       SUCCESS
*****************************************************************************/
KERR_ERR_ENUM FAR PASCAL        KUTFindFitToRectScaleFactors
        (
        LPRECT                  lpSrcRect,      /* (I) pointer to source rectangle */
        LPRECT                  lpDestRect,     /* (I) pointer to destination rectangle */
        LPXYSCALE               lpScale         /* (O) address of returned scale factors */
        );

/*****************************************************************************
* Function:     KUTScaleRect
* Purpose:      Scale the rectangle using the specified
*                       horizontal and vertical scale factors.
* Returns:
*       SUCCESS,
*       KUT_ERR_DIVIDE_BY_ZERO,
*       KUT_ERR_MULTIPLY_OVERFLOW
*****************************************************************************/
KERR_ERR_ENUM FAR PASCAL                KUTScaleRect
        (
        LPRECT                  lpRect,         /* (I/O) rectangle to be scaled */
        LPXYSCALE               lpScale         /* (I) scale factors to be used */
        );

/*****************************************************************************
* Function:     KUTUnscaleRect
* Purpose:      Unscale (scale by 1/scale-factors) the rectangle using
*                       the specified horizontal and vertical scale factors.
* Returns:
*       SUCCESS,
*       KUT_ERR_DIVIDE_BY_ZERO,
*       KUT_ERR_MULTIPLY_OVERFLOW
*****************************************************************************/
KERR_ERR_ENUM FAR PASCAL                KUTUnscaleRect
        (
        LPRECT                  lpRect,         /* (I/O) rectangle to be un-scaled */
        LPXYSCALE               lpScale         /* (I) scale factors to be used */
        );

/*****************************************************************************
* Function:     KUTScalePoint
* Purpose:      Scale the coordinates of a point by the specified
*                       horizontal and vertical scale factors.
* Returns:
*       SUCCESS,
*       KUT_ERR_DIVIDE_BY_ZERO,
*       KUT_ERR_MULTIPLY_OVERFLOW
*****************************************************************************/
KERR_ERR_ENUM FAR PASCAL                KUTScalePoint
        (
        LPPOINT                 lpPoint,        /* (I/O) point to be scaled */
        LPXYSCALE               lpScale         /* (I) scale factors to be used */
        );

/*****************************************************************************
* Function:     KUTUnscalePoint
* Purpose:      Unscale (scale by 1/scale-factor) the coordinates of a point
*                       by the specified horizontal and vertical scale factors.
* Returns:
*       SUCCESS,
*       KUT_ERR_DIVIDE_BY_ZERO,
*       KUT_ERR_MULTIPLY_OVERFLOW
*****************************************************************************/
KERR_ERR_ENUM FAR PASCAL                KUTUnscalePoint
        (
        LPPOINT                 lpPoint,        /* (I/O) point to be un-scaled */
        LPXYSCALE               lpScale         /* (I) scale factors to be used */
        );

/*****************************************************************************
* Function:     KUTRotateRect
* Purpose:      Rotate the rectangle using the specified rotation factor.
* Returns:
*       SUCCESS,
*       KUT_ERR_BAD_ROTATION
*****************************************************************************/
KERR_ERR_ENUM FAR PASCAL                KUTRotateRect
        (
        LPRECT                  lpRect,         /* (I/O) rectangle to be rotated */
        int                             wRotation       /* (I) rotation to be used (degrees) */
        );

/*****************************************************************************
* Function:     KUTUnrotateRect
* Purpose:      Unrotate the rectangle using the specified
*                       the specified rotation factor.
* Returns:
*       SUCCESS,
*       KUT_ERR_BAD_ROTATION
*****************************************************************************/
KERR_ERR_ENUM FAR PASCAL                KUTUnrotateRect
        (
        LPRECT                  lpRect,         /* (I/O) rectangle to be un-rotated */
        int                             wRotation       /* (I) rotation originally used (degrees) */
        );

/*****************************************************************************
* Function:     KUTRotatePoint
* Purpose:      Rotate the given point by the specified rotation factor.
* Returns:
*       SUCCESS,
*       KUT_ERR_BAD_ROTATION
*****************************************************************************/
KERR_ERR_ENUM FAR PASCAL                KUTRotatePoint
        (
        LPPOINT                 lpPoint,        /* (I/O) point to be rotated */
        int                             wRotation       /* (I) rotation to be used (degrees) */
        );

/*****************************************************************************
* Function:     KUTUnrotatePoint
* Purpose:      Unrotate the given point by the specified rotation factor.
* Returns:
*       SUCCESS,
*       KUT_ERR_BAD_ROTATION
*****************************************************************************/
KERR_ERR_ENUM FAR PASCAL                KUTUnrotatePoint
        (
        LPPOINT         lpPoint,                /* (I/O) point to be un-rotated */
        int                     wRotation               /* (I) rotation originally used (degrees) */
        );

/*****************************************************************************
* Function:     KUTMultiplyByRatio
* Purpose:      Multiply a number by a given ratio.
* Returns:
*       SUCCESS
*       KUT_ERR_DIVIDE_BY_ZERO,
*       KUT_ERR_MULTIPLY_OVERFLOW
*****************************************************************************/
KERR_ERR_ENUM FAR PASCAL                KUTMultiplyByRatio
        (
        int                     factor,                 /* (I) input data */
        WORD            wNumerator,             /* (I) ratio's numerator */
        WORD            wDenominator,   /* (I) ratio's denominator */
        LPINT           lpResult                /* (O) address of returned result */
        );

/*****************************************************************************
* Function:     KUTRoundDown
* Purpose:      Round down x to a multiple of Mx.
* Returns:      Rounded down value.
*****************************************************************************/
int FAR PASCAL  KUTRoundDown
        (
        int                             x,                      /* (I) number to be rounded down */
        int                             Mx                      /* (I) desired x divisor */
        );

/*****************************************************************************
* Function:     KUTRoundUp
* Purpose:      Round up x to a multiple of Mx.
* Returns:      Rounded up value.
*****************************************************************************/
int FAR PASCAL  KUTRoundUp
        (
        int                             x,                      /* (I) number to be rounded up */
        int                             Mx                      /* (I) desired x divisor */
        );

/*****************************************************************************
* Function:     KUTRoundedDivide
* Purpose:      Divide and apply rounding to the quotient as specified
*                       by a flag.  The flag can specify mid-point rounding,
*                       rounding up, or rounding down.
* Returns:      The rounded quotient.
*****************************************************************************/

DWORD FAR PASCAL        KUTRoundedDivide
        (
        DWORD           dwNumerator,    /* (I) numerator */
        DWORD           dwDenominator,  /* (I) denominator */
        int                     nRoundingFlag   /* (I) rounding flag:
                                                                                         < 0  --> round down
                                                                                        == 0  --> midpoint round
                                                                                         > 0  --> round up */
        );

/*****************************************************************************
* Function:     KUTBlowupRect
* Purpose:      Inflate the given rectangle so that its x-coordinates are
*                       multiples of Mx and its y-coordinates are multiples of My.
* Returns:      lpRect
*****************************************************************************/
LPRECT FAR PASCAL       KUTBlowupRect
        (
        LPRECT                  lpRect,         /* (I/O) rectangle to be inflated */
        int                             Mx,                     /* (I)   desired x-coordinate divisor */
        int                             My                      /* (I)   desired y-coordinate divisor */
        );

/*****************************************************************************
* Function:     KUTAdjustRatio
* Purpose:      Adjust the ratio defined by the given numerator and
*                       denominator so that the numerator and denominator are
*                       both less than or equal to the given maximum.
* Returns:      A DWORD whose HIWORD is the numerator
*                       and LOWORD is the denominator.
*****************************************************************************/
DWORD FAR PASCAL        KUTAdjustRatio
        (
        DWORD           dwNumerator,    /* (I) given numerator */
        DWORD           dwDenominator,  /* (I) given denominator */
        WORD            wMaxInteger,    /* (I) max for numerator & denominator */
        int                     nRelation               /*      -1 ==> new ratio must be <= old ratio,
                                                                        +1 ==> new ratio must be >= old ratio,
                                                                         0 ==> don't care. */
        );

/*****************************************************************************
* Function:     KUTCompareRatio
* Purpose:      Compare the given decRatios.
* Returns:
*        1      ==> decRatio1 >  decRatio2
*        0      ==> decRatio1 == decRatio2
*       -1      ==> decRatio1 <  decRatio2
*****************************************************************************/
int FAR PASCAL  KUTCompareRatio
        (
        WORD            wNumerator1,    /* (I) numerator for ratio1*/
        WORD            wDenominator1,  /* (I) denominator for ratio1 */
        WORD            wNumerator2,    /* (I) numerator for ratio2 */
        WORD            wDenominator2   /* (I) denominator for ratio2 */
        );

/*****************************************************************************
* Function:     KUTFindFastScaleFactor
* Purpose:      Find the FAST scale factor that best approximates the given
*                       scale factor.  Note that if we are scaling up (enlarging)
*                       then the HW is equally slow for all scale factors, so don't
*                       bother computing fast scale factors and return the original
*                       scale factors.
* Returns:      SUCCESS
*****************************************************************************/
KERR_ERR_ENUM FAR PASCAL                KUTFindFastScaleFactor
        (
        LPXYSCALE               lpScale         /* (I/O) addr of returnd scale factor */
        );

/*****************************************************************************
* Function:     KUTAdjustScaleFactors
* Purpose:      Adjust the given scale factors to be in the range 1 to 32.
* Returns:      SUCCESS
*****************************************************************************/
KERR_ERR_ENUM FAR PASCAL        KUTAdjustScaleFactors
        (
        LPXYSCALE               lpScale         /* (I/O) pointer to scale factors */
        );

/*****************************************************************************
* Function:     KUTLcm
* Purpose:      Compute the Least Common Multiple of the 2 input arguments.
* Returns:      LCM
*****************************************************************************/
WORD FAR PASCAL KUTLcm
        (
        WORD            wM,             /* (I) First number */
        WORD            wN              /* (I) Second number */
        );

/*****************************************************************************
* Function:     KUTGcd
* Purpose:      Calculate the Greatest Common Denominator (GCD) of two numbers.
* Returns:      GCD
*****************************************************************************/
WORD FAR PASCAL KUTGcd
        (
        WORD    wA,                     /* (I) First number */
        WORD    wB                      /* (I) Second number */
        );


/*****************************************************************************
**                                                                                                                                                      **
**                              Memory allocation/deallocation functions                                        **
**                                                                                                                                                      **
*****************************************************************************/

/*****************************************************************************
* Function:     KUTGlobalAllocLocked
* Purpose:      Allocate and lock a memory block as specified by the given
*                       flags.  The allocated memory block size, address and handle
*                       are returned to the user.
* Returns:
*       SUCCESS,
*       KUT_ERR_NO_GLOBAL_MEMORY,
*       KUT_ERR_GLOBAL_LOCK_FAILED
*****************************************************************************/
KERR_ERR_ENUM FAR PASCAL        KUTGlobalAllocLocked
        (
        WORD                    wFlags,         /* (I) allocation flags passed to GlobalAlloc */
        WORD                    wPitch,         /* (I) desired pitch in bytes */
        WORD                    wLength,        /* (I) desired length */
        LPWORD                  lpwLength,      /* (O) address of returned length */
        BOOL                    bExactSize,     /* (I) must be exact size specified? */
        BOOL                    bExactLoc,      /* (I) must be exactly in specified location? */
        HANDLE FAR              *lphMem,        /* (O) address of returned memory handle */
        LPVOID FAR              *lplpMem        /* (O) address of returned memory pointer */
        );

/*****************************************************************************
* Function:     KUTGlobalFreeLocked
* Purpose:      Unlock and free the memory block allocated
*                       with KUTGlobalAllocLocked.
* Returns:
*       SUCCESS
*****************************************************************************/
KERR_ERR_ENUM FAR PASCAL        KUTGlobalFreeLocked
        (
        HANDLE FAR              *lphMem,        /* (I/O) address of memory handle */
        LPVOID FAR              *lplpMem        /* (I/O) address of memory pointer */
        );



/*****************************************************************************
**                                                                                                                                                      **
**                                              Memory Copy/Access functions                                            **
**                                                                                                                                                      **
*****************************************************************************/

/*****************************************************************************
* Function:     KUTCopyParam
* Purpose:      Copy parameters from the srcParamDesc to the dstParamDesc.
* Returns:
*       SUCCESS,
*       KUT_ERR_READ_ONLY_PARAM
*       KUT_ERR_BAD_PARAM_TYPE
*****************************************************************************/
KERR_ERR_ENUM FAR PASCAL        KUTCopyParam
        (
        LP_KUT_PARAM_DESC       lpSrcDesc,      /* (I)   source parameter descriptor */
        LP_KUT_PARAM_DESC       lpDstDesc       /* (I/O) destination parameter descriptor */
        );

/*****************************************************************************
* Function:     KUTCheckBufDesc
* Purpose:      Check the validity of the buffer descriptor for the given buffer.
* Returns:
*       SUCCESS,
*       KUT_ERR_BAD_CXT_BUF_DESC,
*       KUT_ERR_BAD_CXT_SIZE,
*       KUT_ERR_UNSUPPORTED_HANDLE
*****************************************************************************/
KERR_ERR_ENUM FAR PASCAL        KUTCheckBufDesc
        (
        LPVOID                  lpBuf,          /* (I) pointer to buffer */
        DWORD                   dwBytes,        /* (I) size of buffer */
        LP_KUT_BUF_DESC lpBufDesc       /* (I) pointer to buffer descriptor */
        );

/*****************************************************************************
* Function:     KUTGetBufPtr
* Purpose:      Check the buffer pointer for the given buffer descriptor.
* Returns:
*       SUCCESS,
*       KUT_ERR_BAD_CXT_BUF_DESC
*       KUT_ERR_NOT_IMPLEMENTED
*       KUT_ERR_NULL_POINTER
*****************************************************************************/
KERR_ERR_ENUM FAR PASCAL        KUTGetBufPtr
        (
        LP_KUT_BUF_DESC lpBufDesc,      /* (I) pointer to buffer descriptor */
        LPVOID FAR              *lplpBuf        /* (O) address of returned buffer pointer */
        );


#if defined(UNIX)

/* These functions map directly to standard System V functions; UNIX doesn't
 * have the addressing limitations of the other platforms */
#define KUTMemSet               memset
#define KUTMemCpy               memcpy
#define KUTStrCpy               strcpy                                  
#define KUTStrLen               strlen                                  
#define KUTStrNCpy              strncpy                                 
#define KUTStrCmp               strcmp                                  

#else /* !UNIX */

/*****************************************************************************
* Function:     KUTMemSet
* Purpose:      Fill the destination buffer with the given fill value.
* Returns:      Pointer to destination buffer.
*****************************************************************************/
LPVOID FAR PASCAL       KUTMemSet
(
        LPVOID                  lpDst,          /* (O) pointer to buffer to be filled */
        BYTE                    fillByte,       /* (I) fill byte */
        WORD                    wCount          /* (I) count of bytes to be filled */
);

/*****************************************************************************
* Function:     KUTMemCpy
* Purpose:      Copy given number of bytes from the source to the destination.
* Returns:      Pointer to destination buffer.
*****************************************************************************/
LPVOID FAR PASCAL       KUTMemCpy
(
        LPVOID                  lpDst,          /* (O) pointer to destination buffer */
        LPVOID                  lpSrc,          /* (I) pointer to source buffer */
        WORD                    wCount          /* (I) count of bytes to be copied */
);

/*****************************************************************************
* Function:     KUTStrCpy
* Purpose:      Copy the source string to the destination.
* Returns:      Pointer to destination string.
*****************************************************************************/
LPSTR FAR PASCAL                KUTStrCpy
(
        LPSTR                   lpDst,          /* (O) pointer to destination buffer */
        LPSTR                   lpSrc           /* (I) pointer to source buffer */
);

/*****************************************************************************
* Function:     KUTStrLen
* Purpose:      Get the length of a string
* Returns:      Length of string (not including terminating null character).
*****************************************************************************/
WORD FAR PASCAL         KUTStrLen
(
        LPSTR                   lpStr           /* (I) pointer to string */
);

/*****************************************************************************
* Function:     KUTStrNCpy
* Purpose:      Copy at most N characters of the source string to the destination.
* Returns:      Pointer to destination string.
*****************************************************************************/
LPSTR FAR PASCAL                KUTStrNCpy
(
        LPSTR                   lpDst,          /* (O) pointer to destination buffer */
        LPSTR                   lpSrc,          /* (I) pointer to source buffer */
        WORD                    wCount          /* (I) Number of characters to copy */
);

/*****************************************************************************
* Function:     KUTStrCmp
* Purpose:      Compare string 1 against string 2.
* Returns:      < 0             string 1 less than string 2
*                       = 0             string 1 identical to string 2
*                       > 0             string 1 greater than string 2
*****************************************************************************/
int FAR PASCAL KUTStrCmp
        (
        LPSTR   lpszStr1,                       /* (I) First string */
        LPSTR   lpszStr2                        /* (I) Second string */
        );

#endif

/*****************************************************************************
* Function:     KUTBlt
* Purpose:      Copy a block of bits from one buffer to another (bitblt).  The bits
*                       need not be byte aligned in either buffer.  The original value of
*                       destination bits is preserved for those bits in affected destination
*                       bytes which are outside of the copied bits.
* Notes:        It is currently the responsibility of the caller to ensure
*                       that the destination origin of the copied rectangle lies within the 
*                       destination buffer and that there are not too many lines to fit in
*                       the destination buffer.
*                       lpSrcBuf and/or lpDestBuf may be larger than 64K (huge pointers are
*                       used internally).
* Ops:          KUTBLT_COPY                             Copy the buffer
*                       KUTBLT_REVERSE_LINES    Copy buffer, reversing lines
* Returns:
*       SUCCESS,
*       KUT_ERR_BAD_FUNC_PARAM
*****************************************************************************/

KERR_ERR_ENUM FAR PASCAL        KUTBlt(
        KUTBLT_OP       wOperation,             /* (I) Operation to perform */
        LPBYTE          lpSrcBuf,               /* (I) Source bits */
        WORD            wSrcPitch,              /* (I) Pitch (bytes per line) of input buffer */
        WORD            srcX,                   /* (I) source origin (bits into buffer) */
        WORD            srcY,                   /* (I) source origin (lines into buffer) */
        LPBYTE          lpDestBuf,              /* (O) Destination bits */
        WORD            wDestPitch,             /* (I) Pitch (bytes per line) of output buf */
        WORD            destX,                  /* (I) destination origin (bits into buffer) */
        WORD            destY,                  /* (I) destination origin (lines into buffer) */
        WORD            wWidth,                 /* (I) width of rectangle (in bits) */
        WORD            wHeight);               /* (I) height of rectangle (in bits) */

/*****************************************************************************
* Function:     KUTBitsex
* Purpose:      Bitsex the specified buffer.
*****************************************************************************/
void FAR PASCAL KUTBitsex
        ( 
        LPBYTE lpBits,          /* (I/O) Start of buffer to be bitsexed */
        WORD   wCount           /* (I) Number of bytes to bitsex */
        );

/*****************************************************************************
* Function:     KUTInvertBits
* Purpose:      Invert the bits (change 1's to 0's and vice versa) in the 
*                       specified buffer.
* Notes:        This is the same as inverting the color on a monochrome image.
*****************************************************************************/
void FAR PASCAL KUTInvertBits
        ( 
        LPBYTE lpBits,          /* (I/O) Start of buffer to be inverted */
        WORD   wCount           /* (I) Number of bytes to inverted */
        );

/*****************************************************************************
* Function:     KUTReverseLines
* Purpose:      Reverse the lines in the specified buffer without requiring a temp
*                       buffer to be provided.  This means that in a buffer of N lines, the
*                       1st and Nth lines are swapped, 2nd and (N-1)th lines are swapped,
*                       etc.  
* Notes:        If the buffer has an odd number of lines, then the middle line is
*                       unaffected.
* Returns:      SUCCESS (always)
*****************************************************************************/
KERR_ERR_ENUM FAR PASCAL        KUTReverseLines
        (
        LPBYTE          lpBuffer,               /* (I/O) buffer to reverse */
        WORD            wPitch,                 /* (I) Pitch of each line, in bytes */
        WORD            wLineCount              /* (I) Number of lines in buffer */
        );


/*****************************************************************************
**                                                                                                                                                      **
**                                                      Miscelaneous functions                                                  **
**                                                                                                                                                      **
*****************************************************************************/

/*****************************************************************************
* Function:     KUTGetStatusText
* Purpose:      Retrieve text string associated with a particular error/status code.
* Returns:      SUCCESS
*****************************************************************************/
KERR_ERR_ENUM FAR PASCAL KUTGetStatusText(
        int                             nStatus,        /* (I) status in question */
        LPSTR                   lpMsgBuf,       /* (O) buffer address for returned string */
        int                             nBufSize        /* (I) size of lpMsgBuf */
);

/*****************************************************************************
* Function:     KUTWaitForTicks
* Purpose:      Wait for the specified number of ticks (in milli-seconds).
* Returns:      SUCCESS
*****************************************************************************/
KERR_ERR_ENUM FAR PASCAL        KUTWaitForTicks
        (
        DWORD   dwTicks /* (I) number of ticks (in ms) to remain in function */
        );

/*****************************************************************************
* Function:     KUTTableLookup
* Purpose:      Lookup the value "wValue" in "Table" and return the corresponding
*                       value in the table.  If "wEndOfTable" is encountered in the table
*                       while scanning, then we return "wEOTReturn".
* Returns:      Looked-up value from table corresponding to "wValue", OR
*                       "wEOTReturn" if "wEndOfTable" found before a match is found.
*                       Note: if "wValue" == "wEndOfTable", return "wEOTReturn".
*****************************************************************************/

WORD FAR PASCAL         KUTTableLookup
        (
        KUT_WORD_TABLE  Table[],                /* (I) start of table */
        WORD                    wValue,                 /* (I) value to convert */
        WORD                    wEndOfTable,    /* (I) end of table value */
        WORD                    wEOTReturn              /* (I) end of table reached return value */
        );

#if defined(UNIX)

/* For the purposes of the libraries under UNIX, the CPU is always a 386/486 */
#define KUTGetCpuType() (KUT_CPU_80386 | KUT_CPU_PROTECTED)

#else 

/*****************************************************************************
* Function:     KUTGetCpuType
* Purpose:      Determine the CPU type.
* Returns:      Processor type | Protected mode flag, Decode Using KUT_CPU_* symbols
*****************************************************************************/
WORD FAR PASCAL KUTGetCpuType(void);

#endif


/*****************************************************************************
**                                                                                                                                                      **
**                                                              Mapping functions                                                       **
**                                                                                                                                                      **
*****************************************************************************/

/*
 *      The mapping defined by a mapping structure is constructed as follows:
 *
 *      1.      The image is scaled by the given scale factors.
 *              If xyScale is an XYSCALE structure containing the desired scaling factors,
 *              then the scale mapping is given by
 *
 *                      S(x, y)         = ( (xyScale.X.m * x) / xyScale.X.n,
 *                                                              (xyScale.Y.m * y) / xyScale.Y.n )
 *
 *      2.      The scaled image is rotated clockwise by theta degrees.
 *
 *                      If (x,y)        = (r*cos(alpha), r*sin(alpha)) then the rotation
 *                      mapping is given by
 *
 *                      R(x,y)  = (r*cos(alpha+theta), r*sin(alpha+theta))
 *                                      = (r*(cos(alpha)*cos(theta)-sin(alpha)*sin(theta)),
 *                                                      r*(sin(alpha)*cos(theta)+cos(alpha)*sin(theta)))
 *                                      = (r*((x/r)*cos(theta)-(y/r)*sin(theta)),
 *                                                      r*((y/r)*cos(theta)+(x/r)*sin(theta)))
 *                                      = (x*cos(theta)-y*sin(theta), y*cos(theta))
 *                                      =       (x, y)          for theta = 0,
 *                                              (-y, x)         for theta = 90,
 *                                              (-x, -y)        for theta = 180,
 *                                              (y, -x)         for theta = 270.
 *
 *              Ex:
 *
 *                                                      theta = 0
 *                                        (0,0) ZDDDDDDDDDBDDDDDDDDDDDDDDD> X
 *                                                      3                 3
 *                                                      3                 3
 *                                                      3                 3     sclH
 *                                                      3                 3
 *                                                      3                 3
 *                                                      CDDDDDDDDDY
 *                                                      3        sclW
 *                                                      3
 *                                                      v
 *                                                      Y
 *
 *                                                      theta = 90
 *                (-sclH, 0) ZDDDDDDDDDDDDDDBDDDDDDDDDDDDDDDDDDDDDDDDD> X
 *                                       3                              3
 *                                       3                              3 sclW
 *                                       3                              3
 *                                       @DDDDDDDDDDDDDD4
 *                                                      sclH    3
 *                                                                      3
 *                                                                      3
 *                                                                      3
 *                                                                      v
 *                                                                      Y
 *
 *                                                      theta = 180
 *              (-sclW, -sclH)    ZDDDDDDDDD?
 *                                                3                     3
 *                                                3                     3
 *                                                3                     3 sclH
 *                                                3                     3
 *                                                3                     3
 *                                                @DDDDDDDDDEDDDDDDDDDDDDDDDDDDDDDDDDD> X
 *                                                         sclW 3
 *                                                                      3
 *                                                                      3
 *                                                                      3
 *                                                                      3
 *                                                                      3
 *                                                                      3
 *                                                                      3
 *                                                                      v
 *                                                                      Y
 *
 *
 *                                                      theta = 270
 *                              (0, -sclW)      ZDDDDDDDDDDDDDDD?
 *                                                      3                               3
 *                                                      3                               3 sclW
 *                                                      3                               3
 *                                                      CDDDDDDDDDDDDDDDADDDDDDDDD> X
 *                                                      3               sclH
 *                                                      3
 *                                                      3
 *                                                      3
 *                                                      3
 *                                                      3
 *                                                      3
 *                                                      3
 *                                                      v
 *                                                      Y
 *
 *      3.      The scaled, rotated image is translated so that the upper-left hand
 *              corner of the scaled, rotated image is at the origin.
 *
 *                      If srcW and srcH are the width and height of the ORIGINAL image,
 *                      respectively, and the point (RSLeft, RSTop) is the upper-left-hand
 *                      corner of the rotated scaled image of the rectangle,
 *                      ((0, 0), (srcW, srcH)) then the normalization mapping is given by
 *
 *                      N(x, y) =       (x, y) - (RSLeft, RSTop).
 *
 *              Ex:
 *                                                      theta = 0
 *                                                      ZDDDDDDDDDBDDDDDDDDDDDDDDD> X
 *                                                      3                 3
 *                                                      3                 3
 *                                                      3                 3
 *                                                      3                 3
 *                                                      3                 3
 *                                                      CDDDDDDDDDY
 *                                                      3
 *                                                      3
 *                                                      v
 *                                                      Y
 *
 *                                                      theta = 90
 *                                                      ZDDDDDDDDDDDDDDBDDDDDDDDDD> X
 *                                                      3                          3
 *                                                      3                          3
 *                                                      3                          3
 *                                                      CDDDDDDDDDDDDDDY
 *                                                      3
 *                                                      3
 *                                                      3
 *                                                      3
 *                                                      v
 *                                                      Y
 *
 *                                                      theta = 180
 *                                                      ZDDDDDDDDDDBDDDDDDDDDDDDDD> X
 *                                                      3                  3
 *                                                      3                  3
 *                                                      3                  3
 *                                                      3                  3
 *                                                      3                  3
 *                                                      CDDDDDDDDDDY
 *                                                      3
 *                                                      3
 *                                                      v
 *                                                      Y
 *
 *                                                      theta = 270
 *                                                      ZDDDDDDDDDDDDDDDBDDDDDDDDD> X
 *                                                      3                               3
 *                                                      3                               3
 *                                                      3                               3
 *                                                      CDDDDDDDDDDDDDDDY
 *                                                      3
 *                                                      3
 *                                                      3
 *                                                      3
 *                                                      v
 *                                                      Y
 *
 *      4.      The image is translated so that the upper-left hand corner of the
 *              scaled, rotated, normalized image of the source rectangle is moved
 *              to the point, ptDst and then shifted by the x and y scrolling offsets,
 *              xOffset and yOffset.
 *
 *              If (x0, y0) is the upper-left-hand corner of the rotated, normalized,
 *              scaled image of the source rectangle, then the translation
 *              mapping is given by:
 *
 *                      T(x, y) = (x, y) + (ptDst.x, ptDst.y) - (x0, y0)
 *                                              + (xOffset, yOffset)
 *
 *      The composite mapping from the source coordinate system to the destination
 *      coordinate system is given by
 *
 *              T(N(R(S(x, y)))).
 *
 */


/*****************************************************************************
* Function:     KUTMapCxtSize
* Purpose:      Returns the size of a mapping context structure so that
*                       a client can allocate room for a mapping context.
* Returns:      Size of a mapping context structure.
*****************************************************************************/
WORD FAR PASCAL         KUTMapCxtSize ( void );

/*****************************************************************************
* Function:     KUTMapInit
* Purpose:      Initializes a mapping context structure for ues by a client.
* Returns:      SUCCESS
*****************************************************************************/
KERR_ERR_ENUM FAR PASCAL        KUTMapInit
        (
        LP_KUT_MAPPING  lpMapping       /* (O) ptr to mapping context structure */
        );

/*****************************************************************************
* Function:     KUTMapSetParams
* Purpose:      Set the mapping parameters.
* Returns:
*       SUCCESS,
*       KUT_ERR_BAD_PARAM
*****************************************************************************/
KERR_ERR_ENUM FAR PASCAL        KUTMapSetParams
        (
        LP_KUT_MAPPING          lpMapping,      /* (O) pointer to mapping context */
        LP_KUT_MAPPINGPARMS     lpParms         /* (I) pointer to mapping parameters */
        );

/*****************************************************************************
* Function:     KUTMapGetParams
* Purpose:      Get the mapping parameters from the mapping context.
* Returns:      SUCCESS
*****************************************************************************/
KERR_ERR_ENUM FAR PASCAL        KUTMapGetParams
        (
        LP_KUT_MAPPING          lpMapping,      /* (I) pointer to mapping context */
        LP_KUT_MAPPINGPARMS     lpParms         /* (O) pointer to mapping parameters */
        );

/*****************************************************************************
* Function:     KUTMapLPtToSPt
* Purpose:      Map a Logical Point to a Scaled Point.
* Returns:
*       SUCCESS,
*       KUT_ERR_BAD_MAP_HANDLE
*****************************************************************************/
KERR_ERR_ENUM FAR PASCAL        KUTMapLPtToSPt
        (
        LP_KUT_MAPPING  lpMapping,      /* (I)   pointer to mapping context */
        LPPOINT                 lpPt            /* (I/O) pointer to point to be mapped */
        );

/*****************************************************************************
* Function:     KUTMapSPtToSRNPt
* Purpose:      Map a Scaled Point to a Scaled, Rotated, Normalized Point.
* Returns:
*       SUCCESS,
*       KUT_ERR_BAD_MAP_HANDLE
*****************************************************************************/
KERR_ERR_ENUM FAR PASCAL        KUTMapSPtToSRNPt
        (
        LP_KUT_MAPPING  lpMapping,      /* (I) pointer to mapping context */
        LPPOINT                 lpPt            /* (I/O) pointer to point to be mapped */
        );

/*****************************************************************************
* Function:     KUTMapSRNPtToDPt
* Purpose:      Map a Scaled, Rotated, Normalized Point to a Device Point.
* Returns:
*       SUCCESS,
*       KUT_ERR_BAD_MAP_HANDLE
*****************************************************************************/
KERR_ERR_ENUM FAR PASCAL        KUTMapSRNPtToDPt
        (
        LP_KUT_MAPPING  lpMapping,      /* (I) pointer to mapping context */
        LPPOINT                 lpPt            /* (I/O) pointer to point to be mapped */
        );

/*****************************************************************************
* Function:     KUTMapSPtToLPt
* Purpose:      Map a Scaled Point to a Logical Point.
* Returns:
*       SUCCESS,
*       KUT_ERR_BAD_MAP_HANDLE
*****************************************************************************/
KERR_ERR_ENUM FAR PASCAL        KUTMapSPtToLPt
        (
        LP_KUT_MAPPING  lpMapping,      /* (I) pointer to mapping context */
        LPPOINT                 lpPt            /* (I/O) pointer to point to be mapped */
        );

/*****************************************************************************
* Function:     KUTMapSRNPtToSPt
* Purpose:      Map a Scaled, Rotated, Normalized Point to a Scaled Point.
* Returns:
*       SUCCESS,
*       KUT_ERR_BAD_MAP_HANDLE
*****************************************************************************/
KERR_ERR_ENUM FAR PASCAL        KUTMapSRNPtToSPt
        (
        LP_KUT_MAPPING  lpMapping,      /* (I) pointer to mapping context */
        LPPOINT                 lpPt            /* (I/O) pointer to point to be mapped */
        );

/*****************************************************************************
* Function:     KUTMapDPtToSRNPt
* Purpose:      Map a Device Point to a Scaled, Rotated, Normalized Point.
* Returns:
*       SUCCESS,
*       KUT_ERR_BAD_MAP_HANDLE
*****************************************************************************/
KERR_ERR_ENUM FAR PASCAL        KUTMapDPtToSRNPt
        (
        LP_KUT_MAPPING  lpMapping,      /* (I) pointer to mapping context */
        LPPOINT                 lpPt            /* (I/O) pointer to point to be mapped */
        );

/*****************************************************************************
* Function:     KUTMapLRectToSRect
* Purpose:      Map a Logical Rectangle (in source coordinates)
*                       to a Scaled Rectangle.
* Returns:
*       SUCCESS
*       KUT_ERR_BAD_MAP_HANDLE
*****************************************************************************/
KERR_ERR_ENUM FAR PASCAL        KUTMapLRectToSRect
        (
        LP_KUT_MAPPING  lpMapping,      /* (I)   pointer to mapping context */
        LPRECT                  lpRect          /* (I/O) pointer to a rectangle to be mapped */
        );

/*****************************************************************************
* Function:     KUTMapSRectToSRNRect
* Purpose:      Map a Scaled Rectangle to a Scaled, Rotated, Normalized Rectangle.
* Returns:
*       SUCCESS,
*       KUT_ERR_BAD_MAP_HANDLE
*****************************************************************************/
KERR_ERR_ENUM FAR PASCAL        KUTMapSRectToSRNRect
        (
        LP_KUT_MAPPING  lpMapping,      /* (I) pointer to mapping context */
        LPRECT                  lpRect          /* (I/O) pointer rectangle to be mapped */
        );

/*****************************************************************************
* Function:     KUTMapSRNRectToDRect
* Purpose:      Map a Scaled, Rotated, Normalized Rectangle to a Device Rectangle.
* Returns:
*       SUCCESS,
*       KUT_ERR_BAD_MAP_HANDLE
*****************************************************************************/
KERR_ERR_ENUM FAR PASCAL        KUTMapSRNRectToDRect
        (
        LP_KUT_MAPPING  lpMapping,      /* (I)   pointer to mapping context */
        LPRECT                  lpRect          /* (I/O) pointer to a rectangle to be mapped */
        );

/*****************************************************************************
* Function:     KUTMapDRectToSRNRect
* Purpose:      Map a Device Rectangle to a Scaled, Rotated, Normalized Rectangle.
* Returns:
*       SUCCESS,
*       KUT_ERR_BAD_MAP_HANDLE
*****************************************************************************/
KERR_ERR_ENUM FAR PASCAL        KUTMapDRectToSRNRect
        (
        LP_KUT_MAPPING  lpMapping,      /* (I) pointer to mapping context */
        LPRECT                  lpRect          /* (I/O) pointer to a rectangle to be mapped */
        );

/*****************************************************************************
* Function:     KUTMapSRNRectToSRect
* Purpose:      Map a Scaled, Rotated, Normalized Rectangle to a Scaled Rectangle.
* Returns:
*       SUCCESS,
*       KUT_ERR_BAD_MAP_HANDLE
*****************************************************************************/
KERR_ERR_ENUM FAR PASCAL        KUTMapSRNRectToSRect
        (
        LP_KUT_MAPPING  lpMapping,      /* (I) pointer to mapping context */
        LPRECT                  lpRect          /* (I/O) pointer rectangle to be mapped */
        );

/*****************************************************************************
* Function:     KUTMapSRectToLRect
* Purpose:      Map a Scaled Rectangle to a Logical Rectangle (in source
*                       coordinates).
* Returned value
*       SUCCESS,
*       KUT_ERR_BAD_MAP_HANDLE
*****************************************************************************/
KERR_ERR_ENUM FAR PASCAL        KUTMapSRectToLRect
        (
        LP_KUT_MAPPING  lpMapping,      /* (I) pointer to mapping context */
        LPRECT                  lpRect          /* (I/O) pointer to a rectangle to be mapped */
        );


/*****************************************************************************
**                                                                                                                                                      **
**                                                              Alarm functions                                                         **
**                                                                                                                                                      **
*****************************************************************************/


/*****************************************************************************
* Function:     KUTAlarmSet
* Purpose:      Set the timeout period for the alarm.  This is the first function
*                       called when setting up an alarm.
* Returns:      SUCCESS
* History:      AFW - 08/18/89 - created
*****************************************************************************/
KERR_ERR_ENUM FAR PASCAL KUTAlarmSet(
        LP_KUT_ALARM    lpAlarm,
        WORD                    wTimeoutInSeconds);

/*****************************************************************************
* Function:     KUTAlarmStart
* Purpose:      Start or restart the alarm ticking.  The alarm must already have 
*                       been set with KUTAlarmSet().  If the alarm is already running, 
*                       then the alarm is restarted.
* Returns:      SUCCESS or
*                       KUT_ERR_ALARM_NOT_SET --> alarm not previously set
* History:      AFW - 08/18/89 - created
*****************************************************************************/
KERR_ERR_ENUM FAR PASCAL KUTAlarmStart(LP_KUT_ALARM     lpAlarm);

/*****************************************************************************
* Function:     KUTAlarmStop
* Purpose:      Stop the alarm ticking.
* Returns:      SUCCESS
* History:      AFW - 08/18/89 - created
*****************************************************************************/
KERR_ERR_ENUM FAR PASCAL        KUTAlarmStop(LP_KUT_ALARM       lpAlarm);

/*****************************************************************************
* Function:     KUTAlarmCheck
* Purpose:      Check the alarm status.  The alarm should have been previously
*                       started with KUTAlarmStart().
* Returns:      SUCCESS or
*                       KUT_ERR_ALARM_NOT_SET --> alarm not previously set
*                       KUT_ERR_ALARM_NOT_RUNNING --> alarm not previously started
*                       KUT_ERR_ALARM_TIMEOUT --> alarm has timed out
* History:      AFW - 08/18/89 - created
*****************************************************************************/
KERR_ERR_ENUM FAR PASCAL KUTAlarmCheck(LP_KUT_ALARM     lpAlarm);


/*****************************************************************************
**                                                                                                                                                      **
**                                                              Timer macros                                                            **
**                                                                                                                                                      **
*****************************************************************************/

/*****************************************************************************
*       Usage:  Clients should not access the KUT_TIMER_CXT directly.  All access 
*                       should be through the macros defined below.
*
*       KUT_TIMER_CXT           timerCxt;
*
*       Declarations and code.
*
*       KUTTimerInit(timerCxt);
*       KUTTimerStart(timerCxt);
*
*       Code to be timed.
*
*       KUTTimerCapture(timerCxt);
*       printf("Current time in msecs: %lu\n", KUTTimerCurrentTime(timerCxt));
*       KUTTimerStart(timerCxt);                // This avoids timing the printf stmt
*
*       More code to be timed.
*
*       KUTTimerCapture(timerCxt);
*       KUTTimerStop(timerCxt);
*       printf("Current time in msecs: %lu\n", KUTTimerCurrentTime(timerCxt));
*       printf("Elapsed time in msecs: %lu\n", KUTTimerElapsedTime(timerCxt));
*****************************************************************************/

typedef enum {
        KUTTimerStopped,
        KUTTimerRunning,
} KUT_TIMER_STATE;

typedef struct {
        KUT_TIMER_STATE eTimerState;    /* timer state */
        DWORD                   dwElapsedTime;  /* elapsed time (msec) counted by timer */
        DWORD                   dwStartTime;    /* time (msec) when timer last started */
        DWORD                   dwCurrentTime;  /* current timer count (msec) */
} KUT_TIMER_CXT, FAR *LP_KUT_TIMER_CXT;

#define KUTTimerInit(timerCxt) \
                ( (timerCxt).eTimerState = KUTTimerStopped, \
                (timerCxt).dwElapsedTime = \
                (timerCxt).dwCurrentTime = 0 )

#define KUTTimerStart(timerCxt) \
                ( (timerCxt).eTimerState = KUTTimerRunning, \
                (timerCxt).dwStartTime = KOSGetTickCount() )

#define KUTTimerStop(timerCxt) \
                if ( (timerCxt).eTimerState != KUTTimerStopped ) \
                { \
                        (timerCxt).dwCurrentTime        =       (KOSGetTickCount() - (timerCxt).dwStartTime); \
                        (timerCxt).dwElapsedTime        +=      (timerCxt).dwCurrentTime; \
                        (timerCxt).dwStartTime          =       KOSGetTickCount(); \
                        (timerCxt).eTimerState          = KUTTimerStopped; \
                }

#define KUTTimerCapture(timerCxt) \
                if ( (timerCxt).eTimerState == KUTTimerRunning ) \
                { \
                        (timerCxt).dwCurrentTime        =       (timerCxt).dwStartTime; \
                        (timerCxt).dwStartTime          =       KOSGetTickCount(); \
                        (timerCxt).dwCurrentTime        =       (timerCxt).dwStartTime - (timerCxt).dwCurrentTime; \
                        (timerCxt).dwElapsedTime        +=      (timerCxt).dwCurrentTime; \
                }

#define KUTTimerElapsedTime(timerCxt)   ( (timerCxt).dwElapsedTime )

#define KUTTimerCurrentTime(timerCxt)   ( (timerCxt).dwCurrentTime )

/***** Macro to convert the DOS timer ticks used by the CHUCKER to time */
/***** DPCB commands to milli-seconds.  (18.2 DOS ticks = 1 second). */

#define KUTTimerCountToTicks(n)         ((10000L * ((DWORD) n)) / 182L)

#endif /* kut_h */
